;	DAC Code Test Program.
;	=====================
;	This program tests the waveform sample output code for the 10-bit DAC 1 used in the PIC16F1776 device.
;	The 11 waveform samples are stored in a Bank 0 RAM look-up table, and sample output is timed by a PWM 5 Period interrupt
;	routine running at 100us., resulting in an output frequency of 500Hz.
;	The DAC output is buffered by an on-chip op-amp OPA1, which outputs on pin 3/RA1. This will be fed to an LM13700
;	Operational Transconductance Amplifier for low pass filtering. ( 2 pole Butterworth, Fcuttoff  ~ 2.2KHz.)
;	The samples define a sine wave with 11 samples total, covering the range +90 deg. to -90 deg., 18 deg. apart, stored in GPR Bank 0.
;	A pointer in FSR0<H:L> is used to indirectly access these samples. ( Only FSR0<L> is addressed, with FSR0<H> = h'00'.)
;	The pointer moves continuously up and down the table to output the waveform.
;	The clock Fosc = 32MHz, using an 8MHz crystal, and the x4 PLL.
;
;	Results.
;	-------
;	The use of the instruction "moviw FSR0--/++" proved a disaster, as it was found not to work, not decrementing or incrementing
;	the FSR0 register. The data recovery scheme was changed to taking the sample data from Program Memory via a call, with a pointer
;	value in W, and "brw" plus "retlw" in the subroutine. Slower and less efficient - but it worked immediately.
;
;	Later - for interest and completeness - the complementary instruction "moviw --FSR0"/"moviw ++FSR0" was tried - again without success.
;	The DAC o/p made one step from the initialised value, then froze.
;
;	Some of the original code that was tried remains, commented, as does the Program Memory method.
;	
;	Debug code.
;	----------
;	A number of places have alternative code - commented by "DEBUG.".
;	These may use RC2 or RC3 as o/ps.
	
;	COMPILER FAILURE. REGISTER INLVLE.
;	================  ===============
;	Apparently MPLABX doesn't know this register exists in Bank 7, at h'390'.
;	SEE PAGES 28 (BANK 7), 44 AND 169.
;	It should recognise "INLVE" and "INLVE3" however !!

;   Project Configuration.
;   =====================
    errorlevel -202, -302, -306			; Blocking of some compiler message/error reporting :-
											; 202 - Argument out of range.  Least significant bits used.
											; 302 - Register in operand not in bank 0. Ensure that bank bits are correct.
											; 306 - Crossing page boundary - ensure page bits are set.

    LIST b=.4							; LIST options :- tab = 4 spaces.
    radix decimal						; Default radix.
    processor PIC16F1776				; Processor type. (e.g. PIC16F1509)
    #include P16F1776.inc				; Processor register data. (e.g. P16F1509.inc)

;	Configuration bits.
;	==================
; PIC16F1776 Configuration Bit Settings

    __config _CONFIG1, 0x09C2			; FOSC HS, WDTE OFF, PWRTE ON, MCLRE ON, CP OFF, BOREN_OFF, CLKOUTEN OFF, IESO OFF, FCMEN OFF.
    __config _CONFIG2, 0x3FFF			; WRT OFF, PPS1WAY ON, ZCD OFF, PLLEN ON, STVREN ON, BORV LO, LPBOR OFF, LVP ON.

;	Equates.
;	=======
    #include DAC_Code_Equates.inc


;	Program Start.
;	=============
    org 0x000
program_start							; Actions for a Restart :-
    clrf    INTCON						; Disable all Interrupts.
    btfsc   INTCON,GIE					; Was the Disable Interrupts successful?
    bra	    program_start					; If No - jump back and try again,
    goto    main_ini						; else jump to a full Initialisation.


;	Interrupt Service Routine.
;	=========================
    org 0x004							; Interrupt Service routine.
    #include DAC_Code_ISR.inc


;	Main Loop.
;	=========
;	Headed by the Initialisation call, the functional Main Loop starts at "main_start".

;	Initialisation.
;	--------------
main_ini
    movlp   page_0						; ( Page 0.)
    movlb   bank_0						; ( Bank 0.)
    clrf    FSR0H						; ( Indirect Bank 0.)
    clrf    FSR1H
    call    ini_start					; Call to Initialise the Device, Routines and Interrupts used in the program.

;	Main Loop Start.
;	===============
main_start								; Start of the Main Loop.
;	movlb	bank_2				; DEBUG. Toggle RC2 to demonstrate Main Loop function.
;	bcf		LATC,RC2
	nop									; Loop of 16 instructions total, 1us.
	nop
	nop
	nop
	nop
	nop
	nop
;	bsf		LATC,RC2			; DEBUG. Toggle RC2 to demonstrate Main Loop function.
	nop
	nop
	nop
	nop
	nop
	nop
	nop


;	End of Main Loop.
;	----------------
main_loop_end
    bra	    main_start					; Go to the Main Loop start.


;	Sample Data Table.			; DEBUG. For the program memory sample table method.
;	-----------------
;	Sample data defining a sine wave, at 18 deg. intervals.
;sample_data
;	brw
;	retlw	248							; +90 deg., W offset = 0.
;	retlw	242
;	retlw	225
;	retlw	199
;	retlw	165
;	retlw	128							; 0 deg., W offset = 5.
;	retlw	91
;	retlw	57
;	retlw	31
;	retlw	14
;	retlw	8							; -90 deg., W offset = 10.

	
;	Initialisation.
;	--------------
    #include DAC_Code_Ini.inc
    
    end
    